import datetime

from celery import shared_task
from eveuniverse.models import EveConstellation, EveSolarSystem

from allianceauth.eveonline.models import EveFactionInfo

from incursions.helpers import (
    embed_ended, embed_established, embed_established_addendum,
    embed_mobilizing, embed_withdrawing,
)
from incursions.models import Incursion, IncursionInfluence, IncursionsConfig
from incursions.providers import get_incursions_incursions


@shared_task
def update_incursions():
    incursions, response = get_incursions_incursions()
    actives = []
    for incursion in incursions:
        actives.append(incursion['constellation_id'])
        try:
            # Get, because i need to do more steps than an update_or_create would let me
            # Also incursions have no unique id.... wtf ccp
            i = Incursion.objects.get(
                constellation=EveConstellation.objects.get_or_create_esi(id=incursion['constellation_id'])[0],
                ended_timestamp__isnull=True)
            if incursion['state'] == "established":
                # This is still just an established incursion, nothing to act on
                pass
            elif incursion['state'] == "mobilizing" and i.state != Incursion.States.MOBILIZING:
                i.mobilizing_timestamp = datetime.datetime.strptime(
                    str(response.headers['Last-Modified']), '%a, %d %b %Y %H:%M:%S %Z')
                i.state = Incursion.States.MOBILIZING
                i.save(update_fields=["mobilizing_timestamp", "state"])
            elif incursion['state'] == "withdrawing " and i.state != Incursion.States.WITHDRAWING:
                i.withdrawing_timestamp = datetime.datetime.strptime(
                    str(response.headers['Last-Modified']), '%a, %d %b %Y %H:%M:%S %Z')
                i.state = Incursion.States.WITHDRAWING
                i.save(update_fields=["withdrawing_timestamp", "state"])
            else:
                # ????
                pass
            IncursionInfluence.objects.create(
                incursion=i,
                influence=incursion['influence'],
                timestamp=datetime.datetime.strptime(
                    str(response.headers['Last-Modified']), '%a, %d %b %Y %H:%M:%S %Z')
            )
        except Incursion.DoesNotExist:
            # Create an Incursion
            i = Incursion.objects.create(
                constellation=EveConstellation.objects.get_or_create_esi(id=incursion['constellation_id'])[0],
                faction=EveFactionInfo.objects.get_or_create(faction_id=incursion['faction_id'])[0],
                has_boss=incursion['has_boss'],
                staging_solar_system=EveSolarSystem.objects.get_or_create_esi(
                    id=incursion['staging_solar_system_id'])[0],
                state=incursion['state'],
                type=incursion['type'],
                established_timestamp=datetime.datetime.strptime(
                    str(response.headers['Last-Modified']), '%a, %d %b %Y %H:%M:%S %Z')
            )
            IncursionInfluence.objects.create(
                incursion=i,
                influence=incursion['influence'],
                timestamp=datetime.datetime.strptime(
                    str(response.headers['Last-Modified']), '%a, %d %b %Y %H:%M:%S %Z')

            )
    for ended in Incursion.objects.filter(ended_timestamp__isnull=True).exclude(constellation_id__in=actives):
        # Cant use update here, need to fire signals
        ended.ended_timestamp = datetime.datetime.strptime(
            str(response.headers['Last-Modified']), '%a, %d %b %Y %H:%M:%S %Z')
        ended.state = Incursion.States.ENDED
        ended.save(update_fields=["ended_timestamp", "state"])


@shared_task
def incursion_established(incursion_pk: int):
    incursion = Incursion.objects.get(pk=incursion_pk)
    for webhook in IncursionsConfig.get_solo().status_webhooks.all():
        webhook.send_embed(embed=embed_established(incursion))
        webhook.send_embed(embed=embed_established_addendum(incursion))


@shared_task
def incursion_mobilizing(incursion_pk: int):
    incursion = Incursion.objects.get(pk=incursion_pk)
    for webhook in IncursionsConfig.get_solo().status_webhooks.all():
        webhook.send_embed(embed=embed_mobilizing(incursion))


@shared_task
def incursion_withdrawing(incursion_pk: int):
    incursion = Incursion.objects.get(pk=incursion_pk)
    for webhook in IncursionsConfig.get_solo().status_webhooks.all():
        webhook.send_embed(embed=embed_withdrawing(incursion))


@shared_task
def incursion_ended(incursion_pk: int):
    incursion = Incursion.objects.get(pk=incursion_pk)
    for webhook in IncursionsConfig.get_solo().status_webhooks.all():
        webhook.send_embed(embed=embed_ended(incursion))



import os

from allianceauth import __version__ as aa__version__
from allianceauth.services.hooks import get_extension_logger
from esi.clients import EsiClientProvider

from . import __version__ as i__version__

logger = get_extension_logger(__name__)

SWAGGER_SPEC = os.path.join(os.path.dirname(
    os.path.abspath(__file__)),
    'swagger.json')

APP_INFO_TEXT = f"allianceauth v{aa__version__} & aa-incursions v{i__version__}"

"""
Swagger spec operations:
get_incursions
"""


esi = EsiClientProvider(
    spec_file=SWAGGER_SPEC,
    app_info_text=APP_INFO_TEXT)


def get_incursions_incursions():
    operation = esi.client.Incursions.get_incursions()
    operation.request_config.also_return_response = True
    incursions, response = operation.results()
    return incursions, response



from django.contrib import admin

from allianceauth.services.hooks import get_extension_logger

from .models import Incursion, IncursionsConfig, Webhook

logger = get_extension_logger(__name__)


@admin.register(IncursionsConfig)
class IncursionsConfigAdmin(admin.ModelAdmin):
    filter_horizontal = ["status_webhooks", ]


@admin.register(Incursion)
class IncursionAdmin(admin.ModelAdmin):
    list_display = ["constellation", "state"]
    list_filter = ["state", ]


@admin.register(Webhook)
class WebhookAdmin(admin.ModelAdmin):
    list_display = ("url", )



import logging

from django.db.models.signals import post_save
from django.dispatch import receiver

from incursions.models import Incursion
from incursions.tasks import (
    incursion_ended, incursion_established, incursion_mobilizing,
    incursion_withdrawing,
)

logger = logging.getLogger(__name__)


@receiver(post_save, sender=Incursion)
def incursion(sender, instance: Incursion, created: bool, *args, **kwargs):
    update_fields = kwargs.pop('update_fields', []) or []
    if created is True:
        incursion_established.apply_async(args=[instance.pk])
    elif 'state' not in update_fields:
        return
    else:
        if instance.state == Incursion.States.ESTABLISHED:
            # This should have been handled above?
            return
        if instance.state == Incursion.States.MOBILIZING:
            incursion_mobilizing.apply_async(args=[instance.pk])
            return
        if instance.state == Incursion.States.WITHDRAWING:
            incursion_withdrawing.apply_async(args=[instance.pk])
            return
        if instance.state == Incursion.States.ENDED:
            incursion_ended.apply_async(args=[instance.pk])
            return



from django.apps import AppConfig

from . import __version__


class IncursionsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = "incursions"
    verbose_name = f'AA Incursions v{__version__}'

    def ready(self):
        import incursions.signals  # noqa:F401



# Generated by Django 4.0.10 on 2023-06-19 05:40

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('eveuniverse', '0007_evetype_description'),
    ]

    operations = [
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(max_length=200)),
                ('name', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Destination Webhook',
                'verbose_name_plural': 'Destination Webhooks',
            },
        ),
        migrations.CreateModel(
            name='IncursionsConfig',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('security_high', models.BooleanField(verbose_name='Notify on High Security Incursions')),
                ('security_low', models.BooleanField(verbose_name='Notify on Low Security Incursions')),
                ('security_null', models.BooleanField(verbose_name='Notify on Null Security Incursions')),
                ('status_webhooks', models.ManyToManyField(to='incursions.webhook', verbose_name='Destination Webhook for Incursion Updates')),
            ],
            options={
                'verbose_name': 'AA Incursions Settings',
                'verbose_name_plural': 'AA Incursions Settings',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='Incursion',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('has_boss', models.BooleanField(default=False, verbose_name='Whether the final encounter has boss or not')),
                ('influence', models.FloatField(default=0.0, verbose_name='Influence of this incursion as a float from 0 to 1')),
                ('state', models.CharField(choices=[('established', 'Incursion established'), ('mobilizing', 'Incursion mobilized'), ('withdrawing', 'Incursion withdrawing'), ('ended', 'Ended')], default='established', max_length=50, verbose_name='The state of this incursion')),
                ('type', models.CharField(choices=[('Incursion', 'Sansha Incursion')], default='Incursion', max_length=50, verbose_name='The type of this incursion')),
                ('established_timestamp', models.DateTimeField(blank=True, null=True, verbose_name='')),
                ('mobilizing_timestamp', models.DateTimeField(blank=True, null=True, verbose_name='')),
                ('withdrawing_timestamp', models.DateTimeField(blank=True, null=True, verbose_name='')),
                ('ended_timestamp', models.DateTimeField(blank=True, null=True, verbose_name='')),
                ('constellation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveuniverse.eveconstellation', verbose_name='The constellation id in which this incursion takes place')),
                ('faction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='eveonline.evefactioninfo', verbose_name='The attacking faction’s id')),
                ('infested_solar_systems', models.ManyToManyField(related_name='+', to='eveuniverse.evesolarsystem', verbose_name='A list of infested solar system ids that are a part of this incursion')),
                ('staging_solar_system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveuniverse.evesolarsystem', verbose_name='Staging solar system for this incursion')),
            ],
            options={
                'verbose_name': 'Incursion',
                'verbose_name_plural': 'Incursions',
            },
        ),
    ]



import logging

from discord import AutocompleteContext, Option
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands
from eveuniverse.models import EveConstellation, EveRegion, EveSolarSystem

from django.conf import settings

from incursions import __version__
from incursions.models import Incursion

logger = logging.getLogger(__name__)


class Incursions(commands.Cog):
    """
    Drifter Wormhole Mapping and Management
    From AA-incursions
    """

    def __init__(self, bot):
        self.bot = bot

    incursion_commands = SlashCommandGroup(
        "incursions", "Incursions", guild_ids=[int(settings.DISCORD_GUILD_ID)])

    async def search_solar_systems(self, ctx: AutocompleteContext):
        return list(EveSolarSystem.objects.filter(name__icontains=ctx.value).values_list('name', flat=True)[:10])

    async def search_constellations(self, ctx: AutocompleteContext):
        return list(EveConstellation.objects.filter(name__icontains=ctx.value).values_list('name', flat=True)[:10])

    async def search_regions(self, ctx: AutocompleteContext):
        return list(EveRegion.objects.filter(name__icontains=ctx.value).values_list('name', flat=True)[:10])

    async def search_incursion(self, ctx: AutocompleteContext):
        return list(Incursion.objects.filter(name__icontains=ctx.value).values_list('name', flat=True)[:10])

    @incursion_commands.command(name="about", description="About the Incursion Bot", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def about(self, ctx):
        """
        All about the bot
        """
        embed = Embed(title="AA Incursions")
        embed.description = "https://gitlab.com/tactical-supremacy/aa-incursions"
        embed.url = "https://gitlab.com/tactical-supremacy/aa-incursions"
        embed.set_thumbnail(url="https://images.evetech.net/types/2192/render?size=128")
        embed.set_footer(
            text="Developed by Ariel Rin")
        embed.add_field(
            name="Version", value=f"{__version__}", inline=False
        )

        return await ctx.respond(embed=embed)

    @incursion_commands.command(name="focus", description="Get information on the current focus", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def focus(
        self, ctx,
        constellation=Option(str, "Constellation", autocomplete=search_constellations),
    ):
        return await ctx.respond("Not Yet Implemented")

    @incursion_commands.command(name="set_focus", description="Set the current Focus", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def focus_set(
        self, ctx,
        constellation=Option(str, "Constellation", autocomplete=search_constellations),
    ):
        return await ctx.respond("Not Yet Implemented")

    @incursion_commands.command(name="incursions", description="List active incursions", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def incursions(
        self, ctx,
    ):
        return await ctx.respond("Not Yet Implemented")

    @incursion_commands.command(name="incursion_detail", description="Status of a specific incursion", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def incursion_detail(
        self, ctx,
        incursion=Option(str, "Constellation", autocomplete=search_constellations),
    ):
        return await ctx.respond("Not Yet Implemented")


def setup(bot):
    bot.add_cog(Incursions(bot))



import re

from django.conf import settings


def get_site_url():  # regex sso url
    regex = r"^(.+)\/s.+"
    matches = re.finditer(regex, settings.ESI_SSO_CALLBACK_URL, re.MULTILINE)
    url = "http://"

    for m in matches:
        url = m.groups()[0]  # first match

    return url



import logging
from datetime import datetime, timedelta

from discord import Colour, Embed
from eveuniverse.models import EveSolarSystem

from incursions import __version__
from incursions.models import Incursion
from incursions.static_data import incursion_constellations

from .app_settings import get_site_url

logger = logging.getLogger(__name__)


def embed_base() -> Embed:
    embed = Embed(title="AA Incursions")
    embed.url = f"{get_site_url() }/incursions/"
    embed.set_thumbnail(url="https://images.evetech.net/types/2192/render?size=128")
    embed.set_footer(
        text=f"AA Incursions v{__version__}: Developed by Ariel Rin")
    embed.timestamp = datetime.now()
    return embed


def embed_established(incursion: Incursion) -> Embed:
    embed = embed_base()
    embed.colour = Colour.green()
    embed.title = f"New {incursion.security_string}: {incursion.constellation.name}"
    embed.description = "PSA: Create and use an Insta-Dock Bookmark when warping to the new dockup. Scout the route ahead including the dockup station."
    embed.add_field(name="Region", value=incursion.constellation.eve_region.name, inline=True)
    embed.add_field(name="Constellation", value=incursion.constellation.name, inline=True)
    # newline
    embed.add_field(name="Headquarters",
                    value=incursion_constellations[incursion.constellation.name]["Headquarter"])
    embed.add_field(name="Assault",
                    value=incursion_constellations[incursion.constellation.name]["Assaults"], inline=True)
    embed.add_field(name="Vanguard",
                    value=incursion_constellations[incursion.constellation.name]["Vanguards"], inline=True)
    # newline
    embed.add_field(name="Stations in HQ",
                    value=f"{ EveSolarSystem.objects.get(name=incursion_constellations[incursion.constellation.name]['Headquarter']).eve_stations.count() }", inline=True)
    embed.add_field(name="HS Island?", value=f"{incursion.is_island}", inline=True)
    embed.add_field(name="Jumps from last HQ", value="69420lmao", inline=True)
    # newline
    embed.add_field(name="Suggested Dockup", value="Not Implemented", inline=True)
    embed.add_field(name="Edencom In Spawn", value="Not Implemented", inline=True)
    embed.add_field(name="Trigs In Spawn", value="Not Implemented", inline=True)
    return embed


def embed_established_addendum(incursion: Incursion) -> Embed:
    embed = embed_base()
    embed.colour = Colour.blue()
    embed.title = f"{incursion.constellation.name}"
    embed.add_field(name="Spawn Time",
                    value=f"<t:{ int(incursion.established_timestamp.timestamp()) }:R>", inline=True)
    embed.add_field(name="Despawn Window Starts",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=4)).timestamp()) }:R>", inline=True)
    embed.add_field(name="Despawn Window Ends",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=8)).timestamp()) }:R>", inline=True)
    # newline
    embed.add_field(name="Max Established Time",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=5)).timestamp()) }:R>")
    embed.add_field(name="HQ to Hek",
                    value=f"{incursion.staging_solar_system.jumps_to(destination=EveSolarSystem.objects.get(id=30002053)) }", inline=True)
    embed.add_field(name="HQ to Dodixie",
                    value=f"{incursion.staging_solar_system.jumps_to(destination=EveSolarSystem.objects.get(id=30002659)) }", inline=True)
    # newline
    embed.add_field(name="HQ to Rens",
                    value=f"{incursion.staging_solar_system.jumps_to(destination=EveSolarSystem.objects.get(id=30002510)) }")
    embed.add_field(name="HQ to Jita",
                    value=f"{incursion.staging_solar_system.jumps_to(destination=EveSolarSystem.objects.get(id=30000142)) }", inline=True)
    embed.add_field(name="HQ to Amarr",
                    value=f"{incursion.staging_solar_system.jumps_to(destination=EveSolarSystem.objects.get(id=30002187)) }", inline=True)
    # newline
    embed.add_field(name="Gank Pipes In Route", value="Not Implemented")
    embed.add_field(name="Edencom In Route", value="Not Implemented", inline=True)
    embed.add_field(name="Edencom In Route", value="Not Implemented", inline=True)
    return embed


def embed_mobilizing(incursion: Incursion) -> Embed:
    embed = embed_base()
    embed.colour = Colour.yellow()
    embed.title = f"Mobilizing {incursion.security_string}: {incursion.constellation.name}"
    embed.add_field(name="Estimated Withdrawing Time",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=2)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Estimated Despawn Time",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=3)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Max Spawn Stats",
                    value=f'''
                        {incursion.mobilizing_timestamp - incursion.established_timestamp}
                        {(incursion.mobilizing_timestamp - incursion.established_timestamp)+ timedelta(days=5)} Unused
                        {(incursion.mobilizing_timestamp - incursion.established_timestamp) / timedelta(days=5)}% total possible `established` time used")
                    ''')
    embed.add_field(name="Estimated Respawn Window Opens",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=3, hours=12)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Estimated Respawn Window Closes",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=3, hours=36)).timestamp()) }:R>", inline=False)
    return embed

    # Spawn Mobilizing!
    # Estimated Withdrawing Time
    # Friday, 23 June 2023 21:37 (in a day)
    # Estimated Despawn Time
    # Saturday, 24 June 2023 21:37 (in 2 days)
    # Saturday, June 24, 2023 11:37 (Eve Time)
    # Max Spawn Stats
    # 4 days 2 hours 36 minutes established
    # 21 hours 24 minutes unused
    # 82.17% total possible "established" time used
    # Estimated Respawn Window Opens
    # Sunday, 25 June 2023 09:37 (in 3 days)
    # Estimated Respawn Window Closes
    # Monday, 26 June 2023 09:37 (in 4 days)


def embed_withdrawing(incursion: Incursion) -> Embed:
    embed = embed_base()
    embed.colour = Colour.red()
    embed.title = f"Withdrawing {incursion.security_string}: {incursion.constellation.name}"
    embed.add_field(name="Despawn Before",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=1)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Estimated Respawn Window Opens",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=1, hours=12)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Estimated Respawn Window Closes",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=1, hours=36)).timestamp()) }:R>", inline=False)
    return embed

    # Spawn Withdrawing!
    # Despawn Before
    # Saturday, 24 June 2023 21:40 (2 days ago)
    # Saturday, June 24, 2023 11:40 (Eve Time)
    # Estimated Respawn Window Opens
    # Sunday, 25 June 2023 09:40 (a day ago)
    # Estimated Respawn Window Closes
    # Monday, 26 June 2023 09:40 (2 hours ago)


def embed_ended(incursion: Incursion) -> Embed:
    embed = embed_base()
    embed.colour = Colour.red()
    embed.description = "The next spawn will occur in 12-36 hours."
    embed.title = f"Ended {incursion.security_string}: {incursion.constellation.name}"
    embed.add_field(name="Spawn Window Opens",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=1)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Spawn Window Closes",
                    value=f"<t:{ int((incursion.established_timestamp + timedelta(days=1, hours=12)).timestamp()) }:R>", inline=False)
    embed.add_field(name="Max Spawn Stats",
                    value=f'''
                        {incursion.ended_timestamp - incursion.withdrawing_timestamp} Withdrawing
                        {(incursion.ended_timestamp - incursion.withdrawing_timestamp) + timedelta(days=1)} Unused
                        {(incursion.ended_timestamp - incursion.withdrawing_timestamp) / timedelta(days=1):.2f}% total possible `withdrawing` time used")
                    ''')
    return embed

    # Max Spawn Stats
    # 23 hours 50 minutes withdrawing
    # 10 minutes unused
    # 99.31% total possible "withdrawing" time used
